from argparse import ArgumentParser
from collections import defaultdict
from itertools import product
from pathlib import Path
import yaml

from parser import parse_mother_request


PORTFOLIO_PATH = Path(__file__).resolve().parent / "data_portfolio"
PORTFOLIO_KINDS = ["production", "reduced"]
PORTFOLIO_VERSIONS = [path.name for path in PORTFOLIO_PATH.iterdir()]
GENERATIONS = {
    1: "v0.1.1",
    2: "v1.2.0",
}


def _get_parser() -> ArgumentParser:
    parser = ArgumentParser()
    parser.add_argument(
        "path",
        help="Path for mother request",
        type=str,
    )
    return parser


def get_variables(path: Path | str, must_match: bool):
    df = parse_mother_request(path)
    df = df[["Application", "Request No.", "gsv_param", "opa_variable"]]
    variables = defaultdict(list)
    for _, row in df.iterrows():
        gsv = row["gsv_param"]
        opa = row["opa_variable"]
        app = row["Application"]
        if gsv == opa:
            variables[gsv].append(app)
        else:
            if must_match:
                raise ValueError(
                    "GSV and OPA request have non-matching variables: "
                    f"'{row['gsv_param']}' and '{row['opa_variable']}'."
                )
            variables[gsv].append(app)
            variables[opa].append(app)
    return dict(variables)


def read_data_portfolio(version: str, kind: str):
    if version not in PORTFOLIO_VERSIONS:
        raise ValueError(
            f"Version '{version}' is not valid. "
            f"Please, pass one of '{','.join(PORTFOLIO_VERSIONS)}'."
        )
    if kind not in PORTFOLIO_KINDS:
        raise ValueError(
            f"Kind '{kind}' is not valid. "
            f"Please, pass one of '{','.join(PORTFOLIO_KINDS)}'."
        )
    portfolio_path = PORTFOLIO_PATH / version / kind / "portfolio.yaml"
    if not portfolio_path.exists():
        raise FileNotFoundError(f"Malformed portfolio: {version}.")
    with open(portfolio_path, "r", encoding="utf-8") as rf:
        return yaml.safe_load(rf)


def compare_portfolios():
    portfolios = {}
    for kind, (generation, version) in product(PORTFOLIO_KINDS, GENERATIONS.items()):
        portfolios[(kind, generation)] = (
            read_data_portfolio(version=version, kind=kind)
        )
    return portfolios


if __name__ == "__main__":
    args = _get_parser().parse_args()
    path = Path(args.path)
    variables = get_variables(path, must_match=True)
    portfolios = compare_portfolios()
